from django.contrib.admin.widgets import AdminTimeWidget, AdminDateWidget
from django.forms import TextInput
from django.utils.safestring import mark_safe
from django import forms
from django.utils.translation import ugettext as _


class NumberInput(TextInput):

    input_type = 'number'


#
# Original date widgets with addition html
#
class SuitDateWidget(AdminDateWidget):

    def __init__(self, attrs=None, format=None):
        final_attrs = {'class': 'vDateField input-small',
                       'placeholder': _('Date:')[:-1]}
        if attrs is not None:
            final_attrs.update(attrs)
        super(SuitDateWidget, self).__init__(attrs=final_attrs, format=format)

    def render(self, name, value, attrs=None):
        output = super(SuitDateWidget, self).render(name, value, attrs)
        return mark_safe(
            u'<div class="input-append suit-date">%s<span '
            u'class="add-on"><i class="icon-calendar"></i></span></div>' %
            output)


class SuitTimeWidget(AdminTimeWidget):

    def __init__(self, attrs=None, format=None):
        final_attrs = {'class': 'vTimeField input-small',
                       'placeholder': _('Time:')[:-1]}
        if attrs is not None:
            final_attrs.update(attrs)
        super(SuitTimeWidget, self).__init__(attrs=final_attrs, format=format)

    def render(self, name, value, attrs=None):
        output = super(SuitTimeWidget, self).render(name, value, attrs)
        return mark_safe(
            u'<div class="input-append suit-date suit-time">%s<span '
            u'class="add-on"><i class="icon-time"></i></span></div>' %
            output)


class SuitSplitDateTimeWidget(forms.SplitDateTimeWidget):
    """
    A SplitDateTime Widget that has some admin-specific styling.
    """

    def __init__(self, attrs=None):
        widgets = [SuitDateWidget, SuitTimeWidget]
        forms.MultiWidget.__init__(self, widgets, attrs)

    def format_output(self, rendered_widgets):
        out_tpl = u'<div class="datetime">%s %s</div>'
        return mark_safe(out_tpl % (rendered_widgets[0], rendered_widgets[1]))



import sys
import os
import time

from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler, FileModifiedEvent


class LessCompiler(FileSystemEventHandler):

    def __init__(self, source):
        self.source = source
        FileSystemEventHandler.__init__(self)

    def on_any_event(self, event):
        if '__' not in event.src_path and isinstance(event, FileModifiedEvent):
            if len(sys.argv) < 3:
                destination = self.source.replace('less', 'css')
            else:
                destination = sys.argv[2]

            cmd = 'lessc %s > %s -x' % (source, os.path.abspath(destination))
            print(cmd)
            os.system(cmd)


if __name__ == "__main__":

    if len(sys.argv) < 2:
        sys.stderr.write(
            'Usage: %s source [destination=../css/$1.css]\n' % sys.argv[0])
        sys.exit(1)

    source = os.path.abspath(sys.argv[1])
    event_handler = LessCompiler(source)
    observer = Observer()
    observer.schedule(event_handler, os.path.dirname(source), recursive=True)
    observer.start()
    try:
        while True:
            time.sleep(1)
    except KeyboardInterrupt:
        observer.stop()
    observer.join()



from django import template
from django.contrib import admin
from django.core.handlers.wsgi import WSGIRequest
from django.core.urlresolvers import reverse
from suit import get_config

register = template.Library()


@register.assignment_tag(takes_context=True)
def get_menu(context, request):
    """
    :type request: WSGIRequest
    """
    if not isinstance(request, WSGIRequest):
        return None

    # Try to get app list
    template_response = admin.site.index(request)
    try:
        app_list = template_response.context_data['app_list']
    except Exception:
        return

    return Menu(context, request, app_list).get_app_list()


class Menu(object):
    app_activated = False

    def __init__(self, context, request, app_list):
        self.request = request
        self.app_list = app_list

        # Detect current app, if any
        try:
            self.ctx_app = context['app_label'].lower()
        except Exception:
            self.ctx_app = None

        # Get current model plural name, if any
        try:
            self.ctx_model_plural = context['opts'].verbose_name_plural.lower()
        except Exception:
            self.ctx_model_plural = None

        # Flatten all models from native apps
        self.all_models = [model for app in app_list for model in app['models']]

        # Init config variables
        self.init_config()

        super(Menu, self).__init__()

    def init_config(self):
        self.conf_exclude = get_config('MENU_EXCLUDE')
        self.conf_open_first_child = get_config('MENU_OPEN_FIRST_CHILD')
        self.conf_icons = get_config('MENU_ICONS')
        self.conf_menu_order = get_config('MENU_ORDER')

    def get_app_list(self):

        # Reorder menu first, so new hierarchy is present for marking active
        if self.conf_menu_order:
            self.reorder_apps()

        # Exclude apps
        if self.conf_exclude:
            self.exclude_apps()

        # Add icons and match active
        self.activate_apps()

        return self.app_list

    def activate_apps(self):
        for app in self.app_list:
            app_name = app['name'].lower()

            # Set icon if configured
            if self.conf_icons and app_name in self.conf_icons:
                app['icon'] = self.conf_icons[app_name]

            # Mark as active by url match
            if self.request.path == app['app_url'] and not self.app_activated:
                app['is_active'] = self.app_activated = True

            # Activate and exclude app models
            self.activate_models(app)

            # Set first child url on app unless MENU_OPEN_FIRST_CHILD = False
            if self.conf_open_first_child:
                for app in self.app_list:
                    if 'models' in app and len(app['models']) > 0:
                        app['app_url'] = app['models'][0]['admin_url']

    def activate_models(self, app):
        # Iterate models
        for model in app['models']:
            app_name = app['name'].lower()

            # Exclude if in exclude list
            model_full_name = '%s.%s' % (app_name, self.get_model_name(model))
            if self.conf_exclude and model_full_name in self.conf_exclude:
                app['models'].remove(model)
                continue

            # Mark as active by url or model plural name match
            model['is_active'] = self.request.path == model['admin_url']
            model['is_active'] |= self.ctx_model_plural == model['name'].lower()

            # Mark parent as active too
            if model['is_active'] and not self.app_activated:
                app['is_active'] = True

    def exclude_apps(self):
        for app in self.app_list:
            if self.conf_exclude and app['name'].lower() in self.conf_exclude:
                self.app_list.remove(app)
            continue

    def reorder_apps(self):
        new_apps = []
        for order in self.conf_menu_order:
            final_app = None
            app_name = order[0]
            models_order = order[1] if len(order) > 1 else None

            # If custom app
            if isinstance(app_name, (list, tuple)):
                final_app = self.make_custom_app(app_name)
            else:
                # iterate and match django apps
                for app in self.app_list:
                    if app['name'].lower() == app_name:
                        final_app = app
                        break

            if final_app:
                new_apps.append(final_app)

                if models_order:
                    self.reorder_models(final_app, models_order)

        self.app_list = new_apps

    def reorder_models(self, app, models_order):
        new_models = []
        for model_name in models_order:
            # Custom link
            if isinstance(model_name, (list, tuple)):
                custom_link = self.make_custom_link(model_name)
                if custom_link:
                    new_models.append(custom_link)
                continue

            # Append real model link
            for model in self.all_models:
                if model_name == self.get_model_name(model, '.' in model_name):
                    new_models.append(model)

        app['models'] = new_models

    def make_custom_app(self, app):
        app_len = len(app)
        if app_len < 2:
            raise IndexError(
                'Menu custom app must be list or tuple with at least two '
                'parameters: ('
                '"name", "url", "icon"=None)')

        url = self.reverse_url(app[1])
        custom_app = {'name': app[0], 'app_url': url, 'models': []}

        if app_len >= 3:
            custom_app['icon'] = app[2]

        # Check permissions if provided
        if app_len >= 4:
            if not self.user_has_permission(app[3]):
                return

        return custom_app

    def make_custom_link(self, link):
        app_len = len(link)
        if app_len < 2:
            raise IndexError(
                'Menu custom app must be list or tuple with at least two '
                'parameters: ("name", "url")')

        # Check permissions if provided
        if app_len >= 3:
            if not self.user_has_permission(link[2]):
                return

        url = self.reverse_url(link[1])
        return {'name': link[0], 'admin_url': link[1]}

    def user_has_permission(self, perms):
        perms = perms if isinstance(perms, (list, tuple)) else (perms,)
        return self.request.user.has_perms(perms)

    def get_model_name(self, model, is_full_name=False):
        """
        Get model name by its last part of url
        """
        url_parts = model['admin_url'].rstrip('/').split('/')
        if is_full_name:
            return '.'.join(url_parts[-2:])
        return url_parts[-1]

    def reverse_url(self, url):
        if not url or '/' in url:
            return url
        try:
            return reverse(url)
        except:
            return url



VERSION = '0.1.2'


def default_config():
    return {
        'VERSION': VERSION,

        # configurable
        'ADMIN_NAME': 'Django Suit',
        'HEADER_DATE_FORMAT': 'l, jS F Y',
        'HEADER_TIME_FORMAT': 'H:i',

        # form
        'SHOW_REQUIRED_ASTERISK': True,
        'CONFIRM_UNSAVED_CHANGES': True,

        # menu
        'SEARCH_URL': '/admin/auth/user/',
        'MENU_OPEN_FIRST_CHILD': True,
        'MENU_ICONS': {
            'auth': 'icon-lock',
            'sites': 'icon-leaf',
        },
        # 'MENU_EXCLUDE': ('auth.group',),
        # 'MENU_ORDER': (
        #     ('sites',),
        #     ('auth', ('user','group')),
        # )

        # misc
        'LIST_PER_PAGE': 20
    }


from os import getenv

if getenv('DJANGO_SETTINGS_MODULE', None):

    from django.contrib.admin import ModelAdmin
    from django.conf import settings

    def get_config(param=None):

        config_key = 'SUIT_CONFIG'
        if hasattr(settings, config_key):
            config = getattr(settings, config_key, {})
        else:
            config = default_config()
        if param:
            value = None
            if param in config:
                value = config.get(param)
            if value is None:
                value = default_config().get(param)
            return value
        return config

    # Reverse default actions position
    ModelAdmin.actions_on_top = False
    ModelAdmin.actions_on_bottom = True

    # Set global list_per_page
    ModelAdmin.list_per_page = get_config('LIST_PER_PAGE')
